/* list-mon-spells.h - monster race spell flags
 *
 * Changing flag order will break savefiles. There is a hard-coded limit of
 * 128 flags, due to 16 bytes of storage for lore flags in the savefile. Flags
 * below start from 1 on line 31, so a flag's sequence number is its line
 * number minus 30.
 *
 * mana_cost:  Mana cost of spell
 * d_base:     base desirability for AI.
 * d_summ:     desriability for AI per monster level
 *                  times 0-3 based on number of clear spaces
 * d_hurt:     desirability for AI per monster spell power
 *                  times 0-3 based on damage taken
 * d_mana:     desirability for AI per monster spell power
 *                  times 0-2 based on mana shortage
 * d_esc:      desirability for AI per monster level
 *                  times 0-3 based on fear, and damage taken
 * d_tact:     desirability for AI per monster level, modified
 *                  times 0-3 based on proximity, min_range, and best_range
 * d_res:      category of 'resistability' checked by monster AI
 *                 for purposes of desirability.
 * d_range:    % of spell desirability retained for each step past 'range'
 * spell_range: Actual range of spell
 */



/*  name          d_base   d_hurt    d_esc              d_res      spell_range
               mana     d_summ   d_mana    d_tact              d_range       */
RSF(NONE,        0,  0,  0,   0,   0,	0,   0,              0,    0, 0)
RSF(SHRIEK,      1, 40,  0,   0,   5,	0,   0,              0,  100, 0)
RSF(LASH,        0, 40,  0,   0,   5,	0,   0,              0,    0, 3)
RSF(BOULDER,     0, 40,  0,   0,   5,	0,   0,       LRN_ARCH,  100, 0)
RSF(SHOT,        0, 40,  0,   0,   5,	0,   0,       LRN_ARCH,  100, 0)
RSF(ARROW,       0, 40,  0,   0,   5,	0,   0,       LRN_ARCH,  100, 0)
RSF(BOLT,        0, 40,  0,   0,   5,	0,   0,       LRN_ARCH,  100, 0)
RSF(MISSL,       0, 35,  0,   0,   5,	0,   0,       LRN_ARCH,  100, 0)
RSF(PMISSL,      0, 40,  0,   0,   5,	0,   0,      LRN_PARCH,  100, 0)
RSF(BRTH_ACID,   0, 65,  0,   0,   5,	0,   0,       LRN_ACID,   90, 6)
RSF(BRTH_ELEC,   0, 65,  0,   0,   5,	0,   0,       LRN_ELEC,   90, 6)
RSF(BRTH_FIRE,   0, 65,  0,   0,   5,	0,   0,       LRN_FIRE,   90, 6)
RSF(BRTH_COLD,   0, 65,  0,   0,   5,	0,   0,       LRN_COLD,   90, 6)
RSF(BRTH_POIS,   0, 65,  0,   0,   5,	0,   0,       LRN_POIS,   90, 6)
RSF(BRTH_PLAS,   0, 65,  0,   0,   5,	0,   0,       LRN_PLAS,   90, 6)
RSF(BRTH_LIGHT,  0, 50,  0,   0,   5,	0,   0,      LRN_LIGHT,   90, 6)
RSF(BRTH_DARK,   0, 50,  0,   0,   5,	0,   0,       LRN_DARK,   90, 6)
RSF(BRTH_CONFU,  0, 50,  0,   0,   5,	0,   0,      LRN_CONFU,   90, 6)
RSF(BRTH_SOUND,  0, 50,  0,   0,   5,	0,   0,      LRN_SOUND,   90, 6)
RSF(BRTH_SHARD,  0, 50,  0,   0,   5,	0,   0,      LRN_SHARD,   90, 6)
RSF(BRTH_INER,   0, 50,  0,   0,   5,	0,   0,              0,   90, 6)
RSF(BRTH_GRAV,   0, 50,  0,   0,   5,	0,   0,     LRN_SOUND2,   90, 6)
RSF(BRTH_FORCE,  0, 50,  0,   0,   5,	0,   0,     LRN_SOUND2,   90, 6)
RSF(BRTH_NEXUS,  0, 50,  0,   0,   5,	0,   0,      LRN_NEXUS,   90, 6)
RSF(BRTH_NETHR,  0, 50,  0,   0,   5,	0,   0,      LRN_NETHR,   90, 6)
RSF(BRTH_CHAOS,  0, 50,  0,   0,   5,	0,   0,      LRN_CHAOS,   90, 6)
RSF(BRTH_DISEN,  0, 50,  0,   0,   5,	0,   0,      LRN_DISEN,   90, 6)
RSF(BRTH_TIME,   0, 50,  0,   0,   5,	0,   0,              0,   90, 6)
RSF(BRTH_STORM,  0, 65,  0,   0,   5,	0,   0,      LRN_STORM,   90, 6)
RSF(BRTH_DFIRE,  0, 65,  0,   0,   5,	0,   0,      LRN_DFIRE,   90, 6)
RSF(BRTH_ICE,    0, 65,  0,   0,   5,	0,   0,        LRN_ICE,   90, 6)
RSF(BRTH_ALL,    0, 70,  0,   0,   5,	0,   0,        LRN_ALL,  100, 6)
RSF(XXX1,        0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(BALL_ACID,   4, 50,  0,   0,   0,	0,   0,       LRN_ACID,  100, 0)
RSF(BALL_ELEC,   4, 50,  0,   0,   0,	0,   0,       LRN_ELEC,  100, 0)
RSF(BALL_FIRE,   4, 50,  0,   0,   0,	0,   0,       LRN_FIRE,  100, 0)
RSF(BALL_COLD,   4, 50,  0,   0,   0,	0,   0,       LRN_COLD,  100, 0)
RSF(BALL_POIS,   4, 50,  0,   0,   0,	0,   0,       LRN_POIS,  100, 0)
RSF(BALL_LIGHT,  5, 40,  0,   0,   0,	0,   0,      LRN_LIGHT,  100, 0)
RSF(BALL_DARK,   5, 40,  0,   0,   0,	0,   0,       LRN_DARK,  100, 0)
RSF(BALL_CONFU,  6, 40,  0,   0,   0,	0,   0,      LRN_CONFU,  100, 0)
RSF(BALL_SOUND,  4, 40,  0,   0,   0,	0,   0,      LRN_SOUND,  100, 0)
RSF(BALL_SHARD,  4, 40,  0,   0,   0,	0,   0,      LRN_SHARD,  100, 0)
RSF(BALL_STORM,  5, 40,  0,   0,   0,	0,   0,      LRN_STORM,  100, 0)
RSF(BALL_NETHR,  6, 40,  0,   0,   0,	0,   0,      LRN_NETHR,  100, 0)
RSF(BALL_CHAOS,  7, 40,  0,   0,   0,	0,   0,      LRN_CHAOS,  100, 0)
RSF(BALL_MANA,   5, 40,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(BALL_ALL,    5, 50,  0,   0,   0,	0,   0,        LRN_ALL,  100, 0)
RSF(XXX2,        0, 40,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(BOLT_ACID,   4, 40,  0,   0,   0,	0,   0,       LRN_ACID,  100, 0)
RSF(BOLT_ELEC,   4, 40,  0,   0,   0,	0,   0,       LRN_ELEC,  100, 0)
RSF(BOLT_FIRE,   4, 40,  0,   0,   0,	0,   0,       LRN_FIRE,  100, 0)
RSF(BOLT_COLD,   4, 40,  0,   0,   0,	0,   0,       LRN_COLD,  100, 0)
RSF(BOLT_POIS,   4, 40,  0,   0,   0,	0,   0,       LRN_POIS,  100, 0)
RSF(BOLT_PLAS,   4, 50,  0,   0,   0,	0,   0,       LRN_PLAS,  100, 0)
RSF(BOLT_ICE,    4, 50,  0,   0,   0,	0,   0,        LRN_ICE,  100, 0)
RSF(BOLT_WATER,  5, 35,  0,   0,   0,	0,   0,      LRN_WATER,  100, 0)
RSF(BOLT_NETHR,  5, 35,  0,   0,   0,	0,   0,      LRN_NETHR,  100, 0)
RSF(BOLT_DARK,   4, 40,  0,   0,   0,	0,   0,      LRN_CHAOS,  100, 0)
RSF(XXX3,        0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(BEAM_ELEC,   3, 50,  0,   0,   0,	0,   0,       LRN_ELEC,   90, 6)
RSF(BEAM_ICE,    6, 50,  0,   0,   0,	0,   0,        LRN_ICE,   90, 6)
RSF(BEAM_NETHR,  7, 50,  0,   0,   0,	0,   0,      LRN_NETHR,   95, 6)
RSF(ARC_HFIRE,   7, 50,  0,   0,   0,	0,   0,       LRN_FIRE,   90, 6)
RSF(ARC_FORCE,   5, 40,  0,   0,   0,	0,   0,              0,   90, 6)
RSF(HASTE,       6, 50,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(ADD_MANA,    0, 15,  0,   0,  25,	0,   0,              0,  100, 0)
RSF(HEAL,        0, 10, 20,   0,   0,	0,   0,              0,  100, 0)
RSF(CURE,        3, 50,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(BLINK,       1, 27,  0,   0,   0,  10,  15,              0,  100, 0)
RSF(TPORT,       6, 10,  0,   0,   0,  20,  10,              0,  100, 0)
RSF(XXX4,        0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(TELE_SELF_TO,4, 30,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(TELE_TO,     4, 30,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(TELE_AWAY,   8, 10,  0,   0,   0,  20,  10,              0,  100, 0)
RSF(TELE_LEVEL,  8, 10,  0,   0,   0,  20,  10, LRN_NEXUS_SAVE,  100, 0)
RSF(XXX5,        0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(DARKNESS,    1, 20,  0,   0,   0,	5,   0,              0,  100, 0)
RSF(TRAPS,       2, 25,  0,   0,   0,	5,   0,              0,  100, 0)
RSF(FORGET,      6, 25,  0,   0,   0,	5,   0,       LRN_SAVE,  100, 0)
RSF(DRAIN_MANA,  0, 25,  0,   0,  15,	0,   0,       LRN_MANA,  100, 0)
RSF(DISPEL,      5, 25,  0,   0,   5,	0,   0,              0,  100, 0)
RSF(XXX6,        0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(MIND_BLAST,  3, 30,  0,   0,   0,	0,   0,       LRN_SAVE,  100, 0)
RSF(BRAIN_SMASH, 4, 40,  0,   0,   0,	0,   0,       LRN_SAVE,  100, 0)
RSF(WOUND,       4, 40,  0,   0,   0,	0,   0,       LRN_SAVE,  100, 0)
RSF(XXX7,        0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(SHAPECHANGE, 4, 30,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(XXX8,        0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(XXX9,        0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(HUNGER,      2, 25,  0,   0,   0,	0,   0,       LRN_SAVE,  100, 0)
RSF(XX10,        0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(SCARE,       1, 25,  0,   0,   0,	0,   0,  LRN_FEAR_SAVE,  100, 0)
RSF(BLIND,       3, 30,  0,   0,   0,	0,   0, LRN_BLIND_SAVE,  100, 0)
RSF(CONF,        4, 30,  0,   0,   0,	0,   0, LRN_CONFU_SAVE,  100, 0)
RSF(SLOW,        5, 40,  0,   0,   0,	0,   0,  LRN_FREE_SAVE,  100, 0)
RSF(HOLD,        6, 35,  0,   0,   0,	0,   0,  LRN_FREE_SAVE,  100, 0)
RSF(S_KIN,       12, 0, 15,   0,   0,	0,   0,              0,  100, 0)
RSF(S_XXX1,      0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(S_XXX2,      0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(S_MONSTER,   10, 0, 15,   0,   0,	0,   0,              0,  100, 0)
RSF(S_MONSTERS,  15, 0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(S_XXX3,      0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(S_XXX4,      0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(S_XXX5,      0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(S_ANT,       10, 0, 15,   0,   0,	0,   0,              0,  100, 0)
RSF(S_SPIDER,    12, 0, 15,   0,   0,	0,   0,              0,  100, 0)
RSF(S_HOUND,     14, 0, 15,   0,   0,	0,   0,              0,  100, 0)
RSF(S_ANIMAL,    15, 0, 15,   0,   0,	0,   0,              0,  100, 0)
RSF(S_XXX6,      0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(S_XXX7,      0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(S_THIEF,     15, 0, 15,   0,   0,	0,   0,              0,  100, 0)
RSF(S_SWAMP,     15, 0, 15,   0,   0,	0,   0,              0,  100, 0)
RSF(S_XXX8,      0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(S_XXX9,      0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(S_XX10,      0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(S_XX11,      0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(S_DRAGON,    14, 0, 15,   0,   0,	0,   0,              0,  100, 0)
RSF(S_HI_DRAGON, 20, 0, 17,   0,   0,	0,   0,              0,  100, 0)
RSF(S_XX12,      0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(S_XX13,      0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(S_DEMON,     14, 0, 15,   0,   0,	0,   0,              0,  100, 0)
RSF(S_HI_DEMON,  20, 0, 17,   0,   0,	0,   0,              0,  100, 0)
RSF(S_XX14,      0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(S_APPROP,    0,  0,  0,   0,   0,	0,   0,              0,  100, 0)
RSF(S_UNDEAD,    12, 0, 15,   0,   0,	0,   0,              0,  100, 0)
RSF(S_HI_UNDEAD, 20, 0, 17,   0,   0,	0,   0,              0,  100, 0)
RSF(S_QUEST,     20, 0, 20,   0,   0,	0,   0,              0,  100, 0)
RSF(S_UNIQUE,    20, 0, 18,   0,   0,	0,   0,              0,  100, 0)
